/* -*-C-*- inch.c */
/*-->inch*/
/**********************************************************************/
/******************************** inch ********************************/
/**********************************************************************/

/***********************************************************************
Convert a value field in s[] of the form

	------		------------------------------
	value			units implied
	------		------------------------------
	#.##bp		big point (1in = 72bp)
	#.##cc		cicero (1cc = 12dd)
	#.##cm		centimeter
	#.##dd		didot point (1157dd = 1238pt)
	#.##in		inch
	#.##mm		millimeter (10mm = 1cm)
	#.##pc		pica (1pc = 12pt)
	#.##pt		point (72.27pt = 1in)
	#.##sp		scaled point (65536sp = 1pt)
	------		------------------------------

to inches, returning it as the function value.  A leading plus or  minus
sign is optional.  The letter case of the dimension name is ignored.  No
space is permitted between the number and the dimension.

An omitted dimension is equivalent to bp, for PostScript compatibility.
***********************************************************************/

#include "headers.h"

RCSID("$Id: inch.c,v 1.1 2004/01/23 17:53:46 beebe Exp beebe $")


#ifdef DVI
void		fatal ARGS((const char *msg_));
#endif /* DVI */

double		inch ARGS((const char *s_));

double
#if STDC
inch(
const char *s
)
#else /* NOT STDC */
inch(s)
const char *s;
#endif /* STDC */
{
    register double	value;
    char*		units;

    value = strtod(s,&units);		/* on return, units -> dimension */

    if ((*units == '\0') || (stricmp(units,"bp") == 0))	/* big points */
	value /= 72.0;
    else if (stricmp(units,"in") == 0)	/* inches */
	/* NO-OP */;
    else if (stricmp(units,"cm") == 0)	/* centimeters */
	value /= 2.54;
    else if (stricmp(units,"pt") == 0)	/* points */
	value /= 72.27;
    else if (stricmp(units,"cc") == 0)	/* cicero */
	value *= 12.0 * (1238.0 / 1157.0) / 72.27;
    else if (stricmp(units,"dd") == 0)	/* didot points */
	value *= (1238.0 / 1157.0) / 72.27;
    else if (stricmp(units,"mm") == 0)	/* millimeters */
	value /= 25.4;
    else if (stricmp(units,"pc") == 0)	/* pica */
	value *= 12.0 / 72.27;
    else if (stricmp(units,"sp") == 0)	/* scaled points */
	value /= (65536.0 * 72.27);
    else				/* unknown units -- error */

#ifdef DVI				/* must be in DVI driver */
    {
	char		message[256];

	(void)sprintf(message,
	    "inch():  Unrecognized dimension string in [%s].\nExpected one of \
bp, cc, cm, dd, in, mm, pc, pt or sp\n",s);
	fatal(message);
    }
#else  /* NOT DVI */
    {
	(void)fprintf(stderr,
	    "inch():  Unrecognized dimension string [%s]\n",s);
	(void)fprintf(stderr,
	    "expected one of bp, cc, cm, dd, in, mm, pc, pt or sp\n");
	exit(EXIT_FAILURE);
    }
#endif /* DVI */

    return(value);
}
